@@ -15,9 +15,15 @@ import numpy; print "numpy", numpy.__version__
 import scipy; print "scipy", scipy.__version__
 import matplotlib; print "matplotlib", matplotlib.__version__
 import pyglet; print "pyglet", pyglet.version
-import psychopy; print "PsychoPy", psychopy.__version__
+# pyo is a new dependency, for sound:
+try: import pyo; print "pyo", '%i.%i.%i' % pyo.getVersion()
+except: print 'pyo [not installed]'
+
+from psychopy import __version__
+print "\nPsychoPy", __version__
 
 win = visual.Window([100,100])#some drivers want a window open first
+print "have shaders:", win._haveShaders
 print "\nOpenGL info:"
 #get info about the graphics card and drivers
 print "vendor:", gl_info.get_vendor()
@@ -27,7 +33,7 @@ print "(Selected) Extensions:"
 extensionsOfInterest=['GL_ARB_multitexture', 
     'GL_EXT_framebuffer_object','GL_ARB_fragment_program',
     'GL_ARB_shader_objects','GL_ARB_vertex_shader',
-    'GL_ARB_texture_non_power_of_two','GL_ARB_texture_float']
+    'GL_ARB_texture_non_power_of_two','GL_ARB_texture_float', 'GL_STEREO']
 for ext in extensionsOfInterest:
     print "\t", bool(gl_info.have_extension(ext)), ext
 #also determine nVertices that can be used in vertex arrays
