@@ -1,37 +1,43 @@
 """Load and play sounds
 
-There are various APIs for this, none of which are perfect. By default PsychoPy will
-look for and use, in this order: ['pygame','pyglet','pyaudio']
-The API chosen will be stored as a string under::
+By default PsychoPy will try to use the following Libs, in this order, for
+sound reproduction but you can alter the order in preferences>general>audioLib:
+    ['pygame', 'pyo']
+If the lib is `pyo` then there is also a choice of the underlying sound driver.
+Under OSX this is currently set to use coreaudio (rather than portaudio) and on
+windows it will attempt to use an ASIO-based driver if found and fall back to
+DirectSound if not. These settings are not currently configurable but let the
+team know if you need that.
 
-    sound.audioAPI
+The sound lib and driver (if lib==pyo) being used will be stored as::
+    `psychopy.sound.audioLib`
+    `psychopy.sound.audioDriver`
 
-and can be set using, e.g.::
+For control of bitrate and buffer size you can call psychopy.sound.init before
+creating your first Sound object::
 
-    sound.setAudioAPI('pyglet')
+    from psychopy import sound
+    sound.init(rate=44100, stereo=True, buffer=128)
+    s1 = sound.Sound('ding.wav')
+
+pyo (a wrapper for portaudio and coreaudio):
+    pros: low latency where drivers support it (on windows you may want to fetch ASIO4ALL)
+    cons: new in PsychoPy 1.76.00
 
 pygame (must be version 1.8 or above):
     pros: The most robust of the API options so far - it works consistently on all platforms
     cons: needs an additional download, poor latencies
 
-pyglet:
-    pros: comes with enthought python and is already the main API for drawing in PsychoPy
-    cons: complex model using event_dispatch, dodgy timing (just on win32?)
-
-pyaudio:
-    pros: relatively low-level wrapper around portAudio
-    cons: needs another download, rather buggy.
-
 """
 # Part of the PsychoPy library
-# Copyright (C) 2012 Jonathan Peirce
+# Copyright (C) 2013 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
-import numpy, threading, time
+import numpy, time, sys
 from os import path
 from string import capitalize
 from sys import platform, exit, stdout
-from psychopy import event, core, logging
+from psychopy import event, core, logging, prefs
 from psychopy.constants import *
 
 if platform=='win32':
@@ -39,35 +45,32 @@ if platform=='win32':
 else:
     mediaLocation=""
 
-preferredAPI = ['pygame','pyglet','pyaudio']
-global audioAPI, Sound
+global audioLib, audioDriver, Sound, init
+global pyoSndServer
+pyoSndServer=None
 Sound = None
-audioAPI=None
+audioLib=None
+audioDriver=None
 
-try:
-    import pyglet
-    import pyglet.media.procedural
-    pyglet.options['audio'] = ('silent')
-    pyglet.options['debug_gl'] = False
-    import pyglet.media#, pyglet.resource
-    import ctypes, math
-    havePyglet=True
-except ImportError:
-    havePyglet=False
+for thisLibName in prefs.general['audioLib']:
+    try:
+        if thisLibName=='pyo':
+            import pyo
+            havePyo = True
+        elif thisLibName=='pygame':
+            import pygame
+            from pygame import mixer, sndarray
+        else:
+            raise ValueError("Audio lib options are currently only 'pyo' or 'pygame', not '%'" %thisLibName)
+    except:
+        logging.warning('%s audio lib was requested but not loaded: %s' %(thisLibName, sys.exc_info()[1]))
+        continue #to try next audio lib
+    #if we got this far we were sucessful in loading the lib
+    audioLib=thisLibName
+    break
 
-try:
-    import pygame
-    from pygame import mixer, sndarray
-    havePygame=True
-except ImportError:
-    havePygame=False
-
-try:
-    import pyaudio
-    pa = pyaudio.PyAudio()
-    havePyaudio=True
-except ImportError:
-    havePyaudio=False
+if audioLib==None:
+    logging.warning('No audio lib could be loaded. Sounds will not be available.')
 
 class _SoundBase:
     """Create a sound object, from one of many ways.
@@ -102,10 +105,10 @@ class _SoundBase:
         self._snd=None
         self.setSound(value=value, secs=secs, octave=octave)
 
-    def setSound(self, value, secs=0.5, octave=4):
+    def setSound(self, value, secs=0.5, octave=4, log=True):
         """Set the sound to be played.
 
-        Often this is not needed byt the user - it is called implicitly during
+        Often this is not needed by the user - it is called implicitly during
         initialisation.
 
         :parameters:
@@ -123,6 +126,8 @@ class _SoundBase:
                 output sounds in the bottom octave (1) and the top
                 octave (8) is generally painful
         """
+        self._snd = None  # Re-init sound to ensure bad values will raise RuntimeError during setting
+
         try:#could be '440' meaning 440
             value = float(value)
         except:
@@ -141,11 +146,15 @@ class _SoundBase:
         elif type(value) in [list,numpy.ndarray]:
             #create a sound from the input array/list
             self._fromArray(value)
+        #did we succeed?
         if self._snd is None:
             raise RuntimeError, "I dont know how to make a "+value+" sound"
-        self.status=NOT_STARTED
+        else:
+            if log and self.autoLog:
+                logging.exp("Set %s sound=%s" %(self.name, value), obj=self)
+            self.status=NOT_STARTED
 
-    def play(self, fromStart=True):
+    def play(self, fromStart=True, log=True):
         """Starts playing the sound on an available channel.
         If no sound channels are available, it will not play and return None.
 
@@ -157,7 +166,7 @@ class _SoundBase:
         """
         pass #should be overridden
 
-    def stop(self):
+    def stop(self, log=True):
         """Stops the sound immediately"""
         pass #should be overridden
 
@@ -168,7 +177,7 @@ class _SoundBase:
         """Returns the current volume of the sound (0.0:1.0)"""
         pass #should be overridden
 
-    def setVolume(self,newVol):
+    def setVolume(self,newVol, log=True):
         """Sets the current volume of the sound (0.0:1.0)"""
         pass #should be overridden
     def _fromFile(self, fileName):
@@ -231,12 +240,9 @@ class SoundPygame(_SoundBase):
             output sounds in the bottom octave (1) and the top
             octave (8) is generally painful
 
-        sampleRate(=44100): only used for sounds using pyglet. Pygame uses one rate for all sounds
-            sample rate for all sounds (once initialised)
+        sampleRate(=44100): If a sound has already been created or if the
 
-        bits(=16): Only 8- and 16-bits supported so far.
-            Only used for sounds using pyglet. Pygame uses the same
-            sample rate for all sounds (once initialised)
+        bits(=16):  Pygame uses the same bit depth for all sounds once initialised
     """
     def __init__(self,value="C",secs=0.5,octave=4, sampleRate=44100, bits=16, name='', autoLog=True):
         """
@@ -257,7 +263,7 @@ class SoundPygame(_SoundBase):
         self._snd=None
         self.setSound(value=value, secs=secs, octave=octave)
 
-    def play(self, fromStart=True):
+    def play(self, fromStart=True, log=True):
         """Starts playing the sound on an available channel.
         If no sound channels are available, it will not play and return None.
 
@@ -269,10 +275,15 @@ class SoundPygame(_SoundBase):
         """
         self._snd.play()
         self.status=STARTED
-    def stop(self):
+        if log and self.autoLog:
+            logging.exp("Sound %s started" %(self.name), obj=self)
+        return self
+    def stop(self, log=True):
         """Stops the sound immediately"""
         self._snd.stop()
         self.status=STOPPED
+        if log and self.autoLog:
+            logging.exp("Sound %s stopped" %(self.name), obj=self)
     def fadeOut(self,mSecs):
         """fades out the sound (when playing) over mSecs.
         Don't know why you would do this in psychophysics but it's easy
@@ -288,9 +299,11 @@ class SoundPygame(_SoundBase):
         """Returns the current volume of the sound (0.0:1.0)"""
         return self._snd.get_volume()
 
-    def setVolume(self,newVol):
+    def setVolume(self,newVol, log=True):
         """Sets the current volume of the sound (0.0:1.0)"""
         self._snd.set_volume(newVol)
+        if log and self.autoLog:
+            logging.exp("Set Sound %s volume=%.3f" %(self.name, newVol), obj=self)
 
     def _fromFile(self, fileName):
 
@@ -334,10 +347,10 @@ class SoundPygame(_SoundBase):
 
         return True
 
-class SoundPyglet(_SoundBase):
+class SoundPyo(_SoundBase):
     """Create a sound object, from one of MANY ways.
     """
-    def __init__(self,value="C",secs=0.5,octave=4, sampleRate=44100, bits=16):
+    def __init__(self,value="C",secs=0.5,octave=4, stereo=True, sampleRate=44100, bits=16, name='', autoLog=True):
         """
         value: can be a number, string or an array.
 
@@ -361,28 +374,28 @@ class SoundPyglet(_SoundBase):
             output sounds in the bottom octave (1) and the top
             octave (8) is generally painful
 
-        sampleRate(=44100): only used for sounds using pyglet. Pygame uses one rate for all sounds
-            sample rate for all sounds (once initialised)
+        sampleRate(=44100): if the psychopy.sound.init() function has been called
+            or if another sound has already been created then this argument will be
+            ignored and the previous setting will be used
 
-        bits(=16): Only 8- and 16-bits supported so far.
-            Only used for sounds using pyglet. Pygame uses the same
-            sample rate for all sounds (once initialised)
+        bits: has no effect for the pyo backend
         """
+        global pyoSndServer
+        if pyoSndServer==None:
+            initPyo(rate=sampleRate)
 
-        self.sampleRate=sampleRate
+        self.sampleRate=pyoSndServer.getSamplingRate()
         self.format = bits
-        self.isStereo = True
+        self.isStereo = stereo
         self.secs=secs
-        self._player=pyglet.media.ManagedSoundPlayer()
-
-        #self._player._eos_action='pause'
-        self._player._on_eos=self._onEOS
+        self.autoLog=autoLog
+        self.name=name
 
         #try to create sound
         self._snd=None
         self.setSound(value=value, secs=secs, octave=octave)
 
-    def play(self, fromStart=True):
+    def play(self, fromStart=True, log=True):
         """Starts playing the sound on an available channel.
         If no sound channels are available, it will not play and return None.
 
@@ -392,42 +405,38 @@ class SoundPyglet(_SoundBase):
         If you call play() whiles something is already playing the sounds will
         be played over each other.
         """
-        self._player.play()
-        pyglet.media.dispatch_events()
+        self._snd.out()
         self.status=STARTED
+        if log and self.autoLog:
+            logging.exp("Sound %s started" %(self.name), obj=self)
+        return self
 
     def _onEOS(self):
-        self._player._playing = False
-        self._player._timestamp = self._player._sources[0].duration
-        self._player.seek(0)
-        self._player.queue(self._snd)
-        self._player._fill_audio()
-        self._player.dispatch_event('on_eos')
+        #ToDo: is an EOS callback supported by pyo?
         self.status=FINISHED
         return True
 
-    def stop(self):
+    def stop(self, log=True):
         """Stops the sound immediately"""
-        self._snd._stop()
+        self._snd.stop()
         self.status=STOPPED
+        if log and self.autoLog:
+            logging.exp("Sound %s stopped" %(self.name), obj=self)
 
     def getDuration(self):
-        s=self._snd
-        if s.duration is not None:
-            duration = s.duration
-        else:
-            duration = len(s._data)/float(s.audio_format.sample_rate)
-            #data are in byte packets so scale for sample_size (probably 2bytes)
-            duration = duration*8/s.audio_format.sample_size/s.audio_format.channels
-        return duration
+        """Return the duration of the sound file
+        """
+        return self._sndTable.getDur()
 
     def getVolume(self):
         """Returns the current volume of the sound (0.0:1.0)"""
-        return self._player.volume
+        #ToDo : get volume for pyo
+        return volume
 
-    def setVolume(self,newVol):
+    def setVolume(self,newVol, log=True):
         """Sets the current volume of the sound (0.0:1.0)"""
-        self._player._set_volume(newVol)
+        #ToDo : set volume for pyo
+        pass
     def _fromFile(self, fileName):
 
         #try finding the file
@@ -439,361 +448,21 @@ class SoundPyglet(_SoundBase):
                 self.fileName=path.join(filePath,fileName+'.wav')
         if self.fileName is None:
             return False
-
-        self._snd = pyglet.media.load(self.fileName, streaming=False)
-        #for files we need to find the length of the file and
-        if self._snd.duration is not None: #will return none if not determined in the file
-            self.secs=self._snd.duration
-        #add to our player queue
-        self._player.queue(self._snd)
-        return True
-
-    def _fromArray(self, thisArray):
-        global _pygletArrSound
-        #get a mixer.Sound object from an array of floats (-1:1)
-
-        #make stereo if mono
-        if self.isStereo and \
-            (len(thisArray.shape)==1 or thisArray.shape[1]<2):
-            tmp = numpy.ones((len(thisArray),2))
-            tmp[:,0] = thisArray
-            tmp[:,1] = thisArray
-            thisArray = numpy.transpose(tmp)#pyglet wants the transpose
-
-        #use pyglet
-        self._snd = _pygletArrSound(data=thisArray, sample_rate=self.sampleRate, sample_size=-self.format)
-        self._player.queue(self._snd)
-        return True
-
-class SoundPyaudio(_SoundBase):
-    """Create a sound object, from one of MANY ways.
-    """
-    def __init__(self,value="C",secs=0.5,octave=4,
-                    sampleRate=44100, bits=16):
-        """
-        value: can be a number, string or an array.
-
-            If it's a number between 37 and 32767 then a tone will be generated at
-            that frequency in Hz.
-            -----------------------------
-            It could be a string for a note ('A','Bfl','B','C','Csh'...)
-            - you may want to specify which octave as well
-            -----------------------------
-            Or a string could represent a filename in the current
-            location, or mediaLocation, or a full path combo
-            -----------------------------
-            Or by giving an Nx2 numpy array of floats (-1:1) you
-            can specify the sound yourself as a waveform
-
-        secs: is only relevant if the value is a note name or
-            a frequency value
-
-        octave: is only relevant if the value is a note name.
-            Middle octave of a piano is 4. NB On most computers you can't hear
-            sound_snds in the bottom octave (1) and the top
-            octave (8) is generally painful
-
-        sampleRate(=44100)
-
-        bits(=16): 8 or 16
-        """
-        global mediaLocation
-
-        if not havePyglet or not havePyaudio:
-            raise ImportError, "pyglet and pyaudio are both needed for this type of sound"
-        self.offsetSamples = -1
-        self.bits = bits
-        self.sampleRate = sampleRate
-        self.channels=2
-        self.finished=False
-        self.chunkSize=2048# 1024 gives buffer underuns on OSX (even at 22kHz)
-        self.volume = 1.0
-
-        self.rawData = None
-        self._thread = PyAudioThread(self, pollingPeriod=0.01)
-        self._thread.start()
-
-        #try to create sound
-        self._snd=None
-        self.setSound(value=value, secs=secs, octave=octave)
-
-        if self.bits==16: paFormat = pyaudio.paInt16
-        elif self.bits==8: paFormat = pyaudio.paInt8
-        else: raise TypeError, "Sounds must be 8, 16bit"
-        self._stream = pa.open(format = paFormat,
-                channels = self.channels,
-                rate = self.sampleRate,
-                input = False, output=True)
-
-    def play(self, startPos = 0):
-        """Starts playing the sound.
-
-        startPos detremines where the sound begins (in secs)
-        """
-        self.finished=False
-        self.setOffset(startPos)
-        self._fillBuffer()#to get the first sample on its way
-
-    def setOffset(self, secs):
-        #self._snd._seek(0)
-        self.offsetSamples = secs*self.sampleRate
-
-    def stop(self):
-        """Stops the sound immediately"""
-        self.offsetSamples=-1
-
-    def getDuration(self):
-        s=self._snd
-        if s.duration is not None:
-            duration = s.duration
-        else:
-            duration = len(s._data)/float(s.audio_format.sample_rate)
-            #data are in byte packets so scale for sample_size (probably 2bytes)
-            duration = duration*8/s.audio_format.sample_size/s.audio_format.channels
-        return duration
-
-    def getVolume(self):
-        """Returns the current volume of the sound (0.0:1.0)"""
-        return self.volume
-
-    def setVolume(self,newVol):
-        """Sets the current volume of the sound (0.0:1.0)"""
-        self.volume = newVol
-    def _fromFile(self, fileName):
-        #try finding the file
-        self.fileName=None
-        for filePath in ['', mediaLocation]:
-            if path.isfile(path.join(filePath,fileName)):
-                self.fileName=path.join(filePath,fileName)
-            elif path.isfile(path.join(filePath,fileName+'.wav')):
-                self.fileName=path.join(filePath,fileName+'.wav')
-        if self.fileName is None:
-            return False
-
         #load the file
-        self._snd = pyglet.media.load(self.fileName, streaming=False)
-        #convert to an array with int8 or int16
-        if self.bits==16:
-            sndArr = numpy.fromstring(self._snd._data,dtype=numpy.int16)
-        elif self.bits==8:
-            sndArr = numpy.fromstring(self._snd._data,dtype=numpy.uint8)
-            #pyaudio want signed ints so convert unit8 to int8
-            sndArr = (sndArr.astype(numpy.int16)-128).astype(int8)
-        sndArr.shape= [len(sndArr)/self.channels, self.channels]
-        #create the sound buffer from this array
-        self._fromArray(sndArr)
+        self._sndTable = pyo.SndTable(self.fileName)
+        self._snd = pyo.TableRead(self._sndTable, freq=self._sndTable.getRate(), loop=0)
         return True
-    def _fromNoteName(self, name, secs, octave):
-        #get a mixer.Sound object from an note name
-        A=440.0
-        thisNote=capitalize(name)
-        stepsFromA = {
-            'C' : -9,
-            'Csh' : -8,
-            'Dfl' : -8,
-            'D' : -7,
-            'Dsh' : -6,
-            'E' : -5,
-            'F' : -4,
-            'Fsh' : -3,
-            'G' : -2,
-            'Gsh' : -1,
-            'A': 0,
-            'Ash':+1,
-            'Bfl': +1,
-            'B': +2,
-            }
-        if thisNote not in stepsFromA.keys():
-            return False
-
-        thisOctave = octave-4
-        thisFreq = A * 2.0**(stepsFromA[thisNote]/12.0) * 2.0**thisOctave
-        self._fromFreq(thisFreq, secs)
-
-    def _fromFreq(self, thisFreq, secs):
-        #get a mixer.Sound object from a frequency
-        nSamples = int(secs*self.sampleRate)
-        outArr = numpy.arange(0.0,1.0, 1.0/nSamples)
-        outArr *= 2*numpy.pi*thisFreq*secs
-        outArr = numpy.sin(outArr)
-        if self.bits==16:
-            self._fromArray( (outArr*32767).astype(numpy.int16) )
-        if self.bits==8:
-            self._fromArray( (outArr*127.5-0.5).astype(numpy.int8) )#the minus 1 gives range -128:127
 
     def _fromArray(self, thisArray):
-        """Expects an array that is already of the correct format (int8 or int16).
-        Will create a second channel if only one is provided.
-        """
-        #make stereo if mono
-        if self.channels==2 and \
-            (len(thisArray.shape)==1 or thisArray.shape[1]<2):
-                thisArray.shape = [len(thisArray),1]
-                thisArray = thisArray.repeat(2,1)#create the second channel
-        self.rawData = thisArray
+        #ToDo: create a pyo sound from an array
+        if self.isStereo:
+            channels=2
+        else:
+            channels=1
+        self._sndTable = pyo.DataTable(size=len(thisArray), init=thisArray.tolist(), chnls=channels)
+        self._snd = pyo.TableRead(self._sndTable, freq=self._sndTable.getRate(), loop=0)
         return True
 
-    def _fillBuffer(self):
-        """a function that can be called repeatedly to provide more data to the
-        stream"""
-
-        #NB chunkSize and getRemainingBytes() both refer to the number of bytes
-        #(not samples) IN EACH CHANNEL (not in total)
-        #ie. represent half the total number of bytes for a stereo source
-
-        if self.offsetSamples==-1 or self.finished:
-            print 'finishedPlaying', self.offsetSamples, self.finished
-            #sound is not playing yet, just return
-            return
-
-        #get the appropriate data from the array
-        if self.bits == 8:#ubyte
-            start = self.offsetSamples
-            end = self.offsetSamples+self.chunkSize#either the chunk or the last sample
-        elif self.bits==16: #signed int16
-            start = self.offsetSamples >> 1#half as many entries for same number of bytes
-            end = (self.offsetSamples+ self.chunkSize) >> 1
-
-        #check if we have that many samples
-        if end>self.rawData.shape[0]:
-            end = self.rawData.shape[0]
-            print 'end, shape', end, self.rawData.shape[0]
-            self.finished=True#flag that this must be the last sample
-        #update next offset position
-        self.offsetSamples+=self.chunkSize
-
-        thisChunk = (self.volume*self.rawData[start:end,:])
-        print start, end, self.rawData.shape, thisChunk.shape
-        data=thisChunk.tostring()
-        # play stream
-        if len(data)==0:
-            self.finished=True
-            return
-
-        #cwrite the data to the stream
-        self._stream.write(data)
-
-
-def initPyaudio():
-    """
-    define a thread for pyaudio event pumping (needed to fill buffers)
-    """
-    class PyAudioThread(threading.Thread):
-        """a thread class to allow PyAudio sounds to play asynchronously"""
-        def __init__(self, sound, pollingPeriod):
-            threading.Thread.__init__ ( self )
-            self.setDaemon(True)
-            self.sound = sound
-            self.pollingPeriod=pollingPeriod
-            self.running = -1
-        def run(self):
-            self.running=1
-            while self.running:
-                #do the data read
-                self.sound._fillBuffer()
-                time.sleep(self.pollingPeriod)#yields to other processes while sleeping
-            self.running=-1 #shows that it is fully stopped
-        def stop(self):
-            if self.running>0:
-                self.running=0 #make a request to stop on next entry
-        def setPollingPeriod(self, period):
-            self.pollingPeriod=period
-
-def initPyglet():
-    """
-    define a thread for pyglet event pumping (needed to fill buffers)
-    """
-    evtDispatchLock = threading.Lock()
-    class _EventDispatchThread(threading.Thread):
-        """a thread that will periodically call to dispatch events
-        """
-        """I've tried doing this in a way that the thread was started and stopped repeatedly
-        (so could be paused while a sound wasn't needed, but never made it work.
-        see sound.py in SVNr85 for the attempt."""
-        def __init__(self, pollingPeriod=0.005):
-            threading.Thread.__init__ ( self )
-            self.pollingPeriod=pollingPeriod
-            self.running = -1
-            core.runningThreads.append(self)
-        def run(self):
-            self.running=1
-            #print 'thread started'
-            while self.running:
-                #print self.pollingPeriod
-                pyglet.media.dispatch_events()
-                time.sleep(self.pollingPeriod)#yeilds to other processes while sleeping
-            #print 'thread stopped'
-            self.running=-1#shows that it is fully stopped
-        def stop(self):
-            if self.running>0:
-                self.running=0#make a request to stop on next entry
-        def setPollingPeriod(self, period):
-            #print 'polling period is now:%.3f' %period
-            self.pollingPeriod=period
-
-
-    global _eventThread, _pygletArrSound
-    if platform=='win32':
-        _eventThread = _EventDispatchThread(pollingPeriod=0.01)
-    else:
-        _eventThread = _EventDispatchThread(pollingPeriod=0.001)#Mac seem to be able to use a shorter refresh safely
-
-    def setEventPollingPeriod(period):
-        """For pylget contexts this sets the frequency that events controlling sound start and stop
-        are processed in seconds.
-
-        A long period (e.g. 0.1s) will allow more time to be spent on drawing functions and computations,
-        whereas a very short time (e.g. 0.0001) will allow more precise starting/stopping of audio stimuli..
-
-        Events will always be polled on every screen refresh anyway, and repeatedly during
-        calls to event.waitKeys() so this command has few effects on anything other than for very
-        precise sounds.
-        """
-        global _eventThread
-        _eventThread.setPollingPeriod(period)
-    def stopEventPolling():
-        """Stop all polling of events in a pyglet context. Events will still be dispatched on every
-        flip of a visual.Window (every 10-15ms depending on frame rate).
-
-        The user can then dispatch events manually using
-        pyglet.event.dispatch_events()
-        """
-        global _eventThread
-        _eventThread.stop()
-    def startEventPolling():
-        """Restart automated event polling if it has been suspended.
-        This call does nothing if the polling had been
-        """
-        global _eventThread
-        if _eventThread.stopping:
-            _eventThread.start()
-
-    class _pygletArrSound(pyglet.media.procedural.ProceduralSource):
-        """
-        Create a pyglet.StaticSource from a numpy array.
-        """
-        def __init__(self, data, sample_rate=22050, sample_size=16):
-            """Array data should be float (-+1.0)
-            sample_size (16 or 8) determines the number of bits used for internal storage"""
-            duration = data.shape[1]/float(sample_rate) #determine duration from data
-            super(_pygletArrSound, self).__init__(duration,sample_rate, abs(sample_size))
-            self.sample_rate = sample_rate
-            self.sample_size=sample_size
-            if abs(sample_size)==8:          #ubyte
-                self.allData = (data*127+127).astype(numpy.uint8)
-            elif abs(sample_size) == 16:      #signed int16
-                self.allData = (data*32767).astype(numpy.int16)
-
-        def _generate_data(self, bytes, offset, volume=1.0):
-            #print 'bps', self._bytes_per_sample, bytes
-            if self.sample_size == 8:#ubyte
-                start = offset
-                samples = bytes
-            else: #signed int16
-                start = (offset >> 1)#half as many entries for same number of bytes
-                samples = (bytes >> 1)
-            return (self.allData[:,start:(start+samples)]).ctypes
-
 def initPygame(rate=22050, bits=16, stereo=True, buffer=1024):
     """If you need a specific format for sounds you need to run this init
     function. Run this *before creating your visual.Window*.
@@ -805,6 +474,9 @@ def initPygame(rate=22050, bits=16, stereo=True, buffer=1024):
 
     For more details see pygame help page for the mixer.
     """
+    global Sound, audioDriver
+    Sound = SoundPygame
+    audioDriver='n/a'
     if stereo==True: stereoChans=2
     else:   stereoChans=0
     if bits==16: bits=-16 #for pygame bits are signed for 16bit, signified by the minus
@@ -818,31 +490,96 @@ def initPygame(rate=22050, bits=16, stereo=True, buffer=1024):
     if setStereo!=2 and stereo==True:
         logging.warn('Requested stereo setting was not possible')
 
-
-def setAudioAPI(api):
-    """Change the API used for the presentation of sounds
-
-        usage:
-            setAudioAPI(api)
-
-        where:
-            api is one of 'pygame','pyglet', pyaudio'
-
+def _bestDriver(devNames, devIDs):
+    """Find ASIO or Windows sound drivers
     """
-    global audioAPI, Sound
-    exec('haveThis=have%s' %api.title())
-    if haveThis:
-        audioAPI=api
-        exec('init%s()' %(API.title()))
-        exec('thisSound= Sound%s' %(API.title()))
-        Sound= thisSound
-    return haveThis
+    preferredDrivers = prefs.general['audioDriver']
+    outputID=None
+    audioDriver=None
+    for prefDriver in preferredDrivers:
+        if prefDriver.lower()=='directsound':
+            prefDriver = 'Primary Sound'
+        #look for that driver in available devices
+        for devN, devString in enumerate(devNames):
+            if prefDriver in devString:
+                audioDriver=devString
+                outputID=devIDs[devN]
+                return audioDriver, outputID #we found an asio driver don'w look for others
+
+def initPyo(rate=44100, stereo=True, buffer=128):
+    """setup the pyo (sound) server
+    """
+    global pyoSndServer, Sound, audioDriver, duplex
+    Sound = SoundPyo
+    #subclass the pyo.Server so that we can insert a __del__ function that shuts it down
+    class Server(pyo.Server):
+        core=core #make libs class variables so they don't get deleted first
+        logging=logging
+        def __del__(self):
+            self.stop()
+            self.core.wait(0.5)#make sure enough time passes for the server to shutdown
+            self.shutdown()
+            self.core.wait(0.5)#make sure enough time passes for the server to shutdown
+            self.logging.debug('pyo sound server shutdown')#this may never get printed
+
+    #check if we already have a server and kill it
+    if globals().has_key('pyoSndServer') and hasattr(pyoSndServer,'shutdown'): #if it exists and isn't None!
+        #this doesn't appear to work!
+        pyoSndServer.stop()
+        core.wait(0.5)#make sure enough time passes for the server to shutdown
+        pyoSndServer.shutdown()
+        pyoSndServer.reinit(sr=rate, nchnls=2, buffersize=buffer, duplex=1, audio=audioDriver)
+        pyoSndServer.boot()
+    else:
+        #create the instance of the server
+        if platform=='win32':
+            #check for output device/driver
+            devNames, devIDs=pyo.pa_get_output_devices()
+            audioDriver,outputID=_bestDriver(devNames, devIDs)
+            if outputID:
+                logging.info('Using sound driver: %s (ID=%i)' %(audioDriver, outputID))
+            else:
+                logging.warning('No audio outputs found (no speakers connected?')
+                return -1
+            #check for valid input (mic)
+            devNames, devIDs = pyo.pa_get_input_devices()
+            junk, inputID=_bestDriver(devNames, devIDs)
+            if inputID:
+                duplex=True
+            else:
+                duplex=False
+        else:#for other platforms set duplex to True
+            audioDriver = prefs.general['audioDriver'][0]
+            duplex=True
+        if platform=='darwin':
+            #for mac we set the backend using the server audio param
+            pyoSndServer = Server(sr=rate, nchnls=2, buffersize=buffer, audio=audioDriver, duplex=duplex)
+        else:
+            #with others we just use portaudio and then set the OutputDevice below
+            pyoSndServer = Server(sr=rate, nchnls=2, buffersize=buffer, duplex=duplex)
+        pyoSndServer.setVerbosity(1)
+        if platform=='win32':
+            pyoSndServer.setOutputDevice(outputID)
+            if inputID:
+                pyoSndServer.setInputDevice(inputID)
+        #do other config here as needed (setDuplex? setOutputDevice?)
+        pyoSndServer.boot()
+    core.wait(0.5)#wait for server to boot before starting te sound stream
+    pyoSndServer.start()
+    logging.debug('pyo sound server started')
+    logging.flush()
+    
+def setaudioLib(api):
+    """DEPRECATED: please use preferences>general>audioLib to determine which audio lib to use"""
+    raise
 
 #initialise it and keep track
-for API in preferredAPI:
-    if setAudioAPI(API):
-        audioAPI=API
-        break#we found one so stop looking
-if audioAPI is None:
+if audioLib is None:
     logging.error('No audio API found. Try installing pygame 1.8+')
+elif audioLib=='pyo':
+    init=initPyo
+    Sound=SoundPyo
+elif audioLib=='pygame':
+    init=initPygame
+    Sound=SoundPygame
 
